/* THIS SAMPLE CODE IS PROVIDED AS IS AND IS SUBJECT TO ALTERATIONS. FUJITSU */
/* MICROELECTRONICS ACCEPTS NO RESPONSIBILITY OR LIABILITY FOR ANY ERRORS OR */
/* ELIGIBILITY FOR ANY PURPOSES.                                             */
/*                 (C) Fujitsu Microelectronics Europe GmbH                  */
/*---------------------------------------------------------------------------*/

#include <mcu.h>
#include "uart.h"       /* include all UART-Functions */


/*===========================================================================*/

/* declarations */

/* global variables */
char line_buffer[255];
char *line_buffer_ptr;

/* constants */
const char ASCII[] = "0123456789ABCDEF";

/* prototypes */

/*===========================================================================*/


/*****************************************************************************/
/* UART0 Communication routines */
/*****************************************************************************/

/*****************************************************************************
 *  DESCRIPTION:    Initializes UART0     (fr IAR UART4)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        None
 *****************************************************************************/

void InitUART0( void )
{
  //DDR0 |=0x1000;
  FM3_MFS0_UART->SCR =  0x80;
  FM3_MFS0_UART->SMR =  0x01;
  FM3_MFS0_UART->SSR =  0x00;
  FM3_MFS0_UART->ESCR = 0x00;
  FM3_MFS0_UART->BGR = (72000000UL/115200UL-1);  //415;  // 115.2kbaud @ 72 MHz
  FM3_MFS0_UART->SCR = 0x03;
  FM3_GPIO->PFR2 |= (1 << 0x01) | (1 << 0x02);
  FM3_GPIO->EPFR07 |= (1 << 6);
  //DDR0 |= (1 << 0x0B);
  //FM3_MFS0_UART->SMR = 0x0d; // enable SOT0, Reset, normal mode
}

/*****************************************************************************
 *  DESCRIPTION:    sends a single character  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/

void putch0 (char ch)		/* sends a char */
{
   while (!(FM3_MFS0_UART->SSR & (1 << 1)));		/* wait for transmit buffer empty 	*/
	  FM3_MFS0_UART->RDR = ch;            /* put ch into buffer			*/
}

/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/

void Uart_puts0(char *buf)              
{
	while (*buf != '\0')
	{ 
      if(*buf == '\n') putch0('\r');
      putch0(*buf++);        /* send every char of string */
    }
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/

char emulated_input0 = 0;
void emulate_getch0(char c)
{
    emulated_input0 = c;
}
char getch0 (void)   
{
    char c;
	while(!(FM3_MFS0_UART->SSR & (1 << 2)) && (emulated_input0 == 0));			/* wait for data in buffer */
	if (emulated_input0 != 0)
	{
	    c = emulated_input0;
	    emulated_input0 =  0;
	    return c;
	}
	if (FM3_MFS0_UART->SSR & 0x28) {	/* overrun or parity error */
	  return 0;//(-1);
	} else

	return (FM3_MFS0_UART->RDR);				/* return char */
}

/*****************************************************************************
 *  DESCRIPTION:    Receives a string until CR from ext UART
 *
 *  PARAMETERS:     (stored in global variable "line_buffer")
 *
 *  RETURNS:        None
 *****************************************************************************/

void receive_line0(void)
{
  unsigned char ch;
  unsigned short i=0;

	do {
		ch = getch0();  
		if((ch == '\r') | (ch=='\n')) break;
		line_buffer[i++] = ch;
	} while(1);
   
   line_buffer[i]='\0';
   line_buffer_ptr = line_buffer + i;
}

/*****************************************************************************
 *  DESCRIPTION:    Receives a string until CR from ext UART + Echo
 *
 *  PARAMETERS:     (stored in global variable "line_buffer")
 *
 *  RETURNS:        None
 *****************************************************************************/

int receive_line_echo0(unsigned int *cnt)
{
  
  unsigned char ch;
  unsigned short i=0;
  //memset(line_buffer,0,sizeof(line_buffer));

	do {
		ch = getch0();             
        putch0(ch);
        if((ch == 13) | (ch==27)) break;
        line_buffer[i++] = ch;
   } while(1);

   line_buffer[i]='\0';
   line_buffer_ptr = line_buffer + i;
   *cnt = i;
   //BufferPtr = (char)  &line_buffer;
   return (int) &line_buffer;
}

/***********************************************************************
 * DESCRIPTION: Scans the string 'line_buffer' for the string 'string'.
 *              Both strings must be null-terminated. 'line_buffer_ptr'
 *              is set appropriately.
 * RETURN VALUE: 1, if string 'string' is contained in string 'line_buffer'
 *                  at its beginning. 'line_buffer_ptr' points to the
 *                  next character of 'line_buffer' after the equal field.
 *               2, if string 'string' was not found in string 'line_buffer'.
 *               0, if strings are identical to the final null character.
 ************************************************************************/

int scan_line0(char *str) 
{
	line_buffer_ptr = line_buffer;

    while((int)*line_buffer_ptr==(int)*str)
	{
		if((int)*str=='\0') return(0);
		line_buffer_ptr++;
		str++;
	}
	
	if((int)*str=='\0') return(1);
	return(2);
}

/*****************************************************************************
 *  DESCRIPTION:    Inputs a x-digit Hex-number (ASCII characters expected)
 *                      ESC aborts input
 *
 *  PARAMETERS:     Number of Hex-digits (e.g. FF = 2 Hex digits)
 *
 *  RETURNS:        Input value or -1 (abort)
 *****************************************************************************/

unsigned long Inputhex0(unsigned char digits)
{
   unsigned long number=0,digit=0;
   unsigned char abort=0,mlt=0,i,key;

   mlt=(4*(digits-1));  /* 20 for 6 hex-digit numers, 4 for 2 hex-digits */
   for (i=0;i<digits;i++)
   {
     digit = -1;     
     while ((digit==-1) & (!abort))     /* input next valid digit */
     {
       key = getch0();             /* wait for next key input (ASCII) */
       putch0(key);
       if (key == 27) abort=1;    /* ESC aborts */
       digit = ASCIItobin(key);   /* convert to number */ 
       if (digit == 0) putch0(8); /* backspace if not valid */
     }
     number+= (digit << mlt);     /* add digit value to number */
     mlt-=4;                      /* next digit shift */
   }

   if (!abort) 
      return number;             /* return input value */
   else
   {
      puts0("\n\n input cancled \n");
      return -1;                /* return abort indicator */
   }
}

/*****************************************************************************/
/* UART4 Communication routines */
/*****************************************************************************/

/*****************************************************************************
 *  DESCRIPTION:    Initializes UART4
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        None
 *****************************************************************************/

void InitUART4( void )
{ 
  //FM3_MFS4_UART->ESCR = 0x00;
  FM3_MFS4_UART->SCR =  0x80;          // UART reset
  FM3_MFS4_UART->SMR =  0x01;          // serial data output enable, operation mode 0
  FM3_MFS4_UART->SSR =  0x00;          // 0x80 = clear receive error flag
  FM3_MFS4_UART->ESCR = 0x00;          
  FM3_MFS4_UART->BGR = (72000000/115200-1) ;  //415;  // 115.2kbaud @ 72 MHz
  //FM3_MFS4_UART->FCR = 0x0003;        // enable FIFO1 and FIFO2 operation
  //FM3_MFS4_UART->FBYTE1 = 0x10;       // FIFO Byte register for transmit
  //FM3_MFS4_UART->FBYTE2 = 0x10;       // FIFO Byte register for receive
  FM3_MFS4_UART->SCR = 0x03;            // RX, TX enable
  FM3_GPIO->PFR0 |= (1 << 0x0A) | (1 << 0x0B);   // P0A->SIN4_0, P0B->SOT4_0
  FM3_GPIO->EPFR08 |= (1 << 6);            // Use SOT4_0
  //PFR0 |= (1 << 0x0D) | (1 << 0x0E);   // P0D->RTS4_0, P0E->CTS4_0  
  //EPFR08 |= (1 << 0);         // Use RTS4_0
  //FM3_MFS4_UART->ESCR = 0x80;          // enable flow control
  
  //DDR0 |= (1 << 0x0B);
  //FM3_MFS4_UART->SMR = 0x0d; // enable SOT1, Reset, normal mode                  
}

/*****************************************************************************
 *  DESCRIPTION:    sends a single character  (no timeout !)
 *
 *  PARAMETERS:     Character
 *
 *  RETURNS:        None
 *****************************************************************************/

void putch4 (char ch)		/* sends a char */
{
   while (!(FM3_MFS4_UART->SSR & (1 << 1)));		/* wait for transmit buffer empty 	*/
	  FM3_MFS4_UART->RDR = ch;            /* put ch into buffer			*/
}

/*****************************************************************************
 *  DESCRIPTION:    sends a complete string (0-terminated) 
 *
 *  PARAMETERS:     Pointer to (constant) file of bytes in mem
 *
 *  RETURNS:        None
 *****************************************************************************/

void puts4(char *buf)              
{
	while (*buf != '\0')
	{ 
      if(*buf == '\n') putch4('\r');
      putch4(*buf++);        /* send every char of string */
    }
}

/*****************************************************************************
 *  DESCRIPTION:    waits for and returns incoming character (no timeout !)
 *
 *  PARAMETERS:     None
 *
 *  RETURNS:        Character or "-1" (Error)
 *****************************************************************************/

char emulated_input4 = 0;
void emulate_getch4(char c)
{
    emulated_input4 = c;
}
char getch4(void)   
{
    char c;
	while(!(FM3_MFS4_UART->SSR & (1 << 2)) && (emulated_input4 == 0));			/* wait for data in buffer */
	if (emulated_input4 != 0)
	{
	    c = emulated_input4;
	    emulated_input4 =  0;
	    return c;
	}
	if (FM3_MFS4_UART->SSR & 0x28) {	/* overrun or parity error */
	  return 0;//(-1);
	} else

	return (FM3_MFS4_UART->RDR);				/* return char */
}

/*****************************************************************************
 *  DESCRIPTION:    Receives a string until CR from ext UART
 *
 *  PARAMETERS:     (stored in global variable "line_buffer")
 *
 *  RETURNS:        None
 *****************************************************************************/

void receive_line4(void)
{
  unsigned char ch;
  unsigned short i=0;

	do {
		ch = getch4();  
		if((ch == '\r') | (ch=='\n')) break;
		line_buffer[i++] = ch;
	} while(1);
   
   line_buffer[i]='\0';
   line_buffer_ptr = line_buffer + i;
}

/*****************************************************************************
 *  DESCRIPTION:    Receives a string until CR from ext UART + Echo
 *
 *  PARAMETERS:     (stored in global variable "line_buffer")
 *
 *  RETURNS:        None
 *****************************************************************************/

int receive_line_echo4(unsigned int *cnt)
{
  
  unsigned char ch;
  unsigned short i=0;
  //memset(line_buffer,0,sizeof(line_buffer));

	do {
		ch = getch4();             
        putch4(ch);
        if((ch == 13) | (ch==27)) break;
        line_buffer[i++] = ch;
   } while(1);

   line_buffer[i]='\0';
   line_buffer_ptr = line_buffer + i;
   *cnt = i;
   //BufferPtr = (char)  &line_buffer;
   return (int) &line_buffer;
}

/***********************************************************************
 * DESCRIPTION: Scans the string 'line_buffer' for the string 'string'.
 *              Both strings must be null-terminated. 'line_buffer_ptr'
 *              is set appropriately.
 * RETURN VALUE: 1, if string 'string' is contained in string 'line_buffer'
 *                  at its beginning. 'line_buffer_ptr' points to the
 *                  next character of 'line_buffer' after the equal field.
 *               2, if string 'string' was not found in string 'line_buffer'.
 *               0, if strings are identical to the final null character.
 ************************************************************************/

int scan_line4(char *str) 
{
	line_buffer_ptr = line_buffer;

    while((int)*line_buffer_ptr==(int)*str)
	{
		if((int)*str=='\0') return(0);
		line_buffer_ptr++;
		str++;
	}
	
	if((int)*str=='\0') return(1);
	return(2);
}

/*****************************************************************************
 *  DESCRIPTION:    Inputs a x-digit Hex-number (ASCII characters expected)
 *                      ESC aborts input
 *
 *  PARAMETERS:     Number of Hex-digits (e.g. FF = 2 Hex digits)
 *
 *  RETURNS:        Input value or -1 (abort)
 *****************************************************************************/

unsigned long Inputhex4(unsigned char digits)
{
   unsigned long number=0,digit=0;
   unsigned char abort=0,mlt=0,i,key;

   mlt=(4*(digits-1));  /* 20 for 6 hex-digit numers, 4 for 2 hex-digits */
   for (i=0;i<digits;i++)
   {
     digit = 0;     
     while ((digit==0) & (!abort))     /* input next valid digit */
     {
       key = getch4();             /* wait for next key input (ASCII) */
       putch4(key);
       if (key == 27) abort=1;    /* ESC aborts */
       digit = ASCIItobin(key);   /* convert to number */ 
       if (digit == 0) putch4(8); /* backspace if not valid */
     }
     number+= (digit << mlt);     /* add digit value to number */
     mlt-=4;                      /* next digit shift */
   }

   if (!abort) 
      return number;             /* return input value */
   else
   {
      puts4("\n\n input cancled \n");
      return 0;                /* return abort indicator */
   }
}

/*****************************************************************************
 *  DESCRIPTION:    sends a x-digit Hex-number (as ASCII charcaters)
 *                  to terminal via ext. UART
 *
 *  PARAMETERS:     Value and number of Hex-digits (e.g. FF = 2 Hex digits)
 *
 *  RETURNS:        NONE
 *****************************************************************************/
void puthex(unsigned long n, unsigned char digits)
{
	unsigned char i,ch,div=0;

	puts0("0x");				/* hex string */
	div=(digits-1) << 2;	/* init shift divisor */

	for (i=0; i<digits; i++) {
	  ch = (n >> div) & 0xF;/* get hex-digit value */
	  putch0(ASCII[ch]);		/* prompt to terminal as ASCII */
	  div-=4;				/* next digit shift */
   }
}
/*****************************************************************************
 *  DESCRIPTION:    Converts ASCII inputs (0..8,a..f,A..F) to num. value
 *
 *  PARAMETERS:     ASCII input
 *
 *  RETURNS:        value or -1 (error)
 *****************************************************************************/

unsigned long ASCIItobin(unsigned char k)
{
  char d=(char) -1;
  if ((k > 47) & (k < 58)) d = k - 48;  /* 0..9 */
  if ((k > 64) & (k < 71)) d = k - 55;  /* A..F */
  if ((k > 96) & (k < 103)) d = k - 87; /* a..f */
  return d;
}

//============================================================================
// ----------------------- End of Procedure ---------------------------------
//============================================================================
