/************************************************************************/
/*   (C) FUJITSU SEMICONDUCTOR LIMITED AND ITS SUBSIDIARIES (FSLGROUP)  */
/*                                                                      */
/* The following software deliverable is subject the readme text        */
/* included in the same directory or parent directory of this software  */
/* and the use agreement as found on FSLGROUPs Website                 */
/* www.fujitsu.com/global/services/microelectronics/product/micom/      */ 
/* regarding this software.                                             */ 
/* If you do not agree with such terms then DO NOT use this software    */
/* Some highlights of the use agreement are as follows:                 */
/* This software is intended for and must only be                       */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the users obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/*                                                                      */
/* FSLGROUP does not warrant that the deliverables do not infringe any  */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* FSLGROUP disclaims all warranties, whether express or implied,       */
/* in particular, but not limited to, warranties of merchantability     */
/* and fitness for a particular purpose                                 */
/*                                                                      */
/* To the maximum extent permitted by applicable law,                   */
/* FSLGROUPs liability is restricted to intentional misconduct         */
/* and gross negligence.                                                */
/*                                                                      */
/* FSLGROUP is not liable for consequential damages.                    */                             
/*                                                                      */
/* (V2.0)                                                               */
/************************************************************************/

#include "mcu.h"




// LED number pattern array (0 - 9)



const unsigned short LEDPATTERN1[10]={0x2008, 0xF408, 0x4808, 0xC008, 0x9408, 0x8208, 0x0208, 0xF008, 0x0008, 0x8008};
const unsigned short LEDPATTERN2[10]={0x0008, 0x007C, 0x0011, 0x0030, 0x0064, 0x0022, 0x0002, 0x0078, 0x0000, 0x0020};

unsigned char count = 0;          // counter for LED displays
signed char count_direction = 1;  // count direction 

// ---------------- wait loop -----------------
void wait (int a)	 
{
  while(a--);
}

/**
 ******************************************************************************
 ** \brief  Main function of project for MB9B520M series.
 **
 ** \param  none
 ** \return uint32_t return value, if needed
 ******************************************************************************/

int32_t main(void)
{
   FM3_GPIO->ADE = 0x0000;
 // initialize I/O-ports
  FM3_GPIO->PFR4  &= 0xFBFF;  // select CPIO function            
  FM3_GPIO->DDR4  &= 0xFBFF;  // Port 4 input (Button INT21_1)
  
  FM3_GPIO->PFR0  &= 0x7FFF;  // select CPIO function            
  FM3_GPIO->DDR0  &= 0x7FFF;  // Port 0 input (Button NMIX/WKUP0)
    
   
  FM3_GPIO->PFR5 &=0xFF87;   // select CPIO function
  FM3_GPIO->PDOR5 &= 0xFF87;  // set pins to turn off leds
  FM3_GPIO->DDR5 |= 0x0078;   // make led pins outputs
  
  FM3_GPIO->PFR3 &=0x01F0;   // select CPIO function
  FM3_GPIO->PDOR3 &= 0x01F0;  // set pins to turn off leds
  FM3_GPIO->DDR3 |= 0xFE0F;   // make led pins outputs

  while(1)
  {
    FM3_GPIO->PDOR5 = 0x78 & LEDPATTERN2[count - ((count / 10) * 10)];                   // set tens' place 
    FM3_GPIO->PDOR3 = 0x07 & LEDPATTERN2[count - ((count / 10) * 10)];                   // set tens' place  
    FM3_GPIO->PDOR3 |= LEDPATTERN1[count / 10];  // set ones' place
    
    wait (1000000);                     // wait some time ....
    
    if (!(!(bFM3_GPIO_PDIR4_PA) && !(bFM3_GPIO_PDIR0_PF))) // both key buttons not pressed?
    {
      if (!(bFM3_GPIO_PDIR4_PA))  // SW3-INT21_1  pressed?
      {
        count_direction = 1;           // count direction downwards 
      }
  
      if (!(bFM3_GPIO_PDIR0_PF))  //  SW2-NMIX/WKUP0 pressed?
      {
        count_direction = -1;            // count diretion upwards
      }
    }
        
    if ((1 == count_direction ) && (count < 99))  // still count upwards?
    {
      count++;                          // count upwards, if < 99
    }
    
    if ((-1 == count_direction ) && (count != 0)) // still count downwards?
    {
      count--;                          // count downwards, if != 0
    }   
    
  }

}
