/************************************************************************/
/*               (C) Fujitsu Semiconductor Europe GmbH                  */
/*                                                                      */
/* The following software deliverable is intended for and must only be  */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the users obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/* FSEU does not warrant that the deliverables do not infringe any      */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* To the maximum extent permitted by applicable law FSEU disclaims all */
/* warranties, whether express or implied, in particular, but not       */
/* limited to, warranties of merchantability and fitness for a          */
/* particular purpose for which the deliverable is not designated.      */
/*                                                                      */
/* To the maximum extent permitted by applicable law, FSEU's liability  */
/* is restricted to intention and gross negligence.                     */
/* FSEU is not liable for consequential damages.                        */
/*                                                                      */
/* (V1.0)                                                               */
/************************************************************************/
/*  Startup for GNU toolchain                                           */
/*  Version     V2.1                                                    */
/*  Date        2011-04-26                                              */
/*  Target-mcu  MB9BF52x                                                 */
/************************************************************************/


/*
// <h> Stack Configuration
//   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

	.equ    Stack_Size, 0x00000400
    .section ".stack", "w"
    .align  3
    .globl  __FM3_stack_mem
    .globl  __FM3_stack_size
__FM3_stack_mem:
    .if     Stack_Size
    .space  Stack_Size
    .endif
    .size   __FM3_stack_mem,  . - __FM3_stack_mem
    .set    __FM3_stack_size, . - __FM3_stack_mem


/*
// <h> Heap Configuration
//   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
// </h>
*/

    .equ    Heap_Size,  0x00000040
    
    .section ".heap", "w"
    .align  3
    .globl  __FM3_heap_start
    .globl  __FM3_heap_end
__FM3_heap_start:
    .if     Heap_Size
    .space  Heap_Size
    .endif
__FM3_heap_end:


/* Vector Table */

    .section ".FM3.interrupt_vector"
    .globl  __FM3_interrupt_vector_cortex_m
    .type   __FM3_interrupt_vector_cortex_m, %object

__FM3_interrupt_vector_cortex_m:
    .long   __FM3_stack                 /* Top of Stack                 */
    .long   __FM3_reset                 /* Reset Handler                */
    .long   NMI_Handler                 /* NMI Handler                  */
    .long   HardFault_Handler           /* Hard Fault Handler           */
    .long   MemManage_Handler           /* MPU Fault Handler            */
    .long   BusFault_Handler            /* Bus Fault Handler            */
    .long   UsageFault_Handler          /* Usage Fault Handler          */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   0                           /* Reserved                     */
    .long   SVC_Handler                 /* SVCall Handler               */
    .long   DebugMon_Handler            /* Debug Monitor Handler        */
    .long   0                           /* Reserved                     */
    .long   PendSV_Handler              /* PendSV Handler               */
    .long   SysTick_Handler             /* SysTick Handler              */

    /* External Interrupts */

    .long	CSV_Handler               /* 0: Clock Super Visor						*/
    .long	SWDT_Handler              /* 1: Software Watchdog Timer					*/
    .long  	LVD_Handler               /* 2: Low Voltage Detector					*/
    .long  	MFT_WG_IRQHandler         /* 3: Wave Form Generator / DTIF				*/
    .long  	INT0_7_Handler            /* 4: External Interrupt Request ch.0 to ch.7	*/
    .long	INT8_15_Handler           /* 5: External Interrupt Request ch.8 to ch.15*/
    .long  	DT_Handler                /* 6: Dual Timer / Quad Decoder				*/
    .long  	MFS0RX_IRQHandler         /* 7: MultiFunction Serial ch.0				*/
    .long  	MFS0TX_IRQHandler         /* 8: MultiFunction Serial ch.0				*/
    .long  	MFS1RX_IRQHandler         /* 9: MultiFunction Serial ch.1				*/
    .long  	MFS1TX_IRQHandler         /* 10: MultiFunction Serial ch.1				*/
    .long  	MFS2RX_IRQHandler         /* 11: MultiFunction Serial ch.2				*/
    .long  	MFS2TX_IRQHandler         /* 12: MultiFunction Serial ch.2				*/
    .long  	MFS3RX_IRQHandler         /* 13: MultiFunction Serial ch.3				*/
    .long  	MFS3TX_IRQHandler         /* 14: MultiFunction Serial ch.3				*/
    .long  	MFS4RX_IRQHandler         /* 15: MultiFunction Serial ch.4				*/
    .long  	MFS4TX_IRQHandler         /* 16: MultiFunction Serial ch.4				*/
    .long  	MFS5RX_IRQHandler         /* 17: MultiFunction Serial ch.5				*/
    .long  	MFS5TX_IRQHandler         /* 18: MultiFunction Serial ch.5				*/
    .long  	MFS6RX_IRQHandler         /* 19: MultiFunction Serial ch.6				*/
    .long  	MFS6TX_IRQHandler         /* 20: MultiFunction Serial ch.6				*/
    .long  	MFS7RX_IRQHandler         /* 21: MultiFunction Serial ch.7				*/
    .long  	MFS7TX_IRQHandler         /* 22: MultiFunction Serial ch.7				*/
    .long  	PPG_Handler               /* 23: PPG									*/
    .long  	TIM_IRQHandler            /* 24: OSC / PLL / Watch Counter				*/
    .long  	ADC0_IRQHandler           /* 25: ADC0									*/
    .long  	ADC1_IRQHandler           /* 26: ADC1									*/
    .long  	ADC2_IRQHandler           /* 27: ADC2									*/
    .long  	MFT_FRT_IRQHandler        /* 28: Free-run Timer							*/
    .long  	MFT_IPC_IRQHandler        /* 29: Input Capture							*/
    .long  	MFT_OPC_IRQHandler        /* 30: Output Compare							*/
    .long  	BT_IRQHandler             /* 31: Base Timer ch.0 to ch.7				*/
    .long  	CAN0_IRQHandler           /* 32: CAN ch.0								*/
    .long  	CAN1_IRQHandler           /* 33: CAN ch.1								*/
    .long  	USBF_Handler              /* 34: USB Function							*/
    .long  	USB_Handler               /* 35: USB Function / USB HOST				*/
    .long  	DummyHandler_1            /* 36: Reserved								*/
    .long  	DummyHandler_2            /* 37: Reserved								*/
    .long  	DMAC0_Handler             /* 38: DMAC ch.0								*/
    .long  	DMAC1_Handler             /* 39: DMAC ch.1								*/
    .long  	DMAC2_Handler             /* 40: DMAC ch.2								*/
    .long  	DMAC3_Handler             /* 41: DMAC ch.3								*/
    .long  	DMAC4_Handler             /* 42: DMAC ch.4								*/
    .long  	DMAC5_Handler             /* 43: DMAC ch.5								*/
    .long  	DMAC6_Handler             /* 44: DMAC ch.6								*/
    .long  	DMAC7_Handler             /* 45: DMAC ch.7								*/
    .long  	DummyHandler_3            /* 46: Reserved								*/
    .long  	DummyHandler_4            /* 47: Reserved								*/

    .size   __FM3_interrupt_vector_cortex_m, . - __FM3_interrupt_vector_cortex_m


    .thumb


/* Reset Handler */

    .section .FM3.reset,"x",%progbits
    .thumb_func
    .globl  __FM3_reset_cortex_m
    .type   __FM3_reset_cortex_m, %function
__FM3_reset_cortex_m:
    .fnstart
.ifdef Debug_RAM
/* this routine is used for .bss sections initialization by RAM debugging "Debug_RAM = 1" */
/* Clear .bss section (Zero init) */
	MOV     R0, #0
	LDR     R1, =_sbss
	LDR     R2, =_ebss
	CMP     R1,R2
	BEQ     start_main
Loop_bss:
	CMP     R1, R2
	BHS		start_main
	STR   	R0, [R1]
	ADD		R1, #4
	BLO     Loop_bss

.else
/* this routine is used for .data and .bss sections initialization by ROM debugging "Debug_RAM = 0" */
init_data:
	LDR		R0, =_sdata
	LDR		R1, =_etext
	LDR		R2, =data_size
	MOV		R3, #0
loop_init_data:
	CMP		R3, R2
	BCS		zero_bss
    LDR		R4,[R1,#0]
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R1, #4
	ADD		R0, #4
	B		loop_init_data

/* Clear .bss section (Zero init) */

zero_bss:
	LDR		R0, =_sbss
	LDR		R2, =bss_size
	MOV		R3, #0
	MOV		R4, #0
loop_zero_bss:
	CMP		R3, R2
	BCS		start_main
	STR		R4,[R0,#0]
	ADD		R3, #4
	ADD		R0, #4
	B		loop_zero_bss
.endif
start_main:
	LDR     R0, =SystemInit
    BLX     R0
    LDR     R0,=main
    BX      R0

    .pool
    .cantunwind
    .fnend
    .size   __FM3_reset_cortex_m,.-__FM3_reset_cortex_m

    .section ".text"

/* Exception Handlers */

    .weak   NMI_Handler
    .type   NMI_Handler, %function
NMI_Handler:
    B       .
    .size   NMI_Handler, . - NMI_Handler

    .weak   HardFault_Handler
    .type   HardFault_Handler, %function
HardFault_Handler:
    B       .
    .size   HardFault_Handler, . - HardFault_Handler

    .weak   MemManage_Handler
    .type   MemManage_Handler, %function
MemManage_Handler:
    B       .
    .size   MemManage_Handler, . - MemManage_Handler

    .weak   BusFault_Handler
    .type   BusFault_Handler, %function
BusFault_Handler:
    B       .
    .size   BusFault_Handler, . - BusFault_Handler

    .weak   UsageFault_Handler
    .type   UsageFault_Handler, %function
UsageFault_Handler:
    B       .
    .size   UsageFault_Handler, . - UsageFault_Handler

    .weak   SVC_Handler
    .type   SVC_Handler, %function
SVC_Handler:
    B       .
    .size   SVC_Handler, . - SVC_Handler

    .weak   DebugMon_Handler
    .type   DebugMon_Handler, %function
DebugMon_Handler:
    B       .
    .size   DebugMon_Handler, . - DebugMon_Handler

    .weak   PendSV_Handler
    .type   PendSV_Handler, %function
PendSV_Handler:
    B       .
    .size   PendSV_Handler, . - PendSV_Handler

    .weak   SysTick_Handler
    .type   SysTick_Handler, %function
SysTick_Handler:
    B       .
    .size   SysTick_Handler, . - SysTick_Handler


/* IRQ Handlers */

    .globl  Default_Handler
    .type   Default_Handler, %function
Default_Handler:
    B       .
    .size   Default_Handler, . - Default_Handler

    .macro  IRQ handler
    .weak   \handler
    .set    \handler, Default_Handler
    .endm

	IRQ		CSV_Handler
    IRQ		SWDT_Handler
    IRQ  	LVD_Handler
    IRQ  	MFT_WG_IRQHandler
    IRQ  	INT0_7_Handler
    IRQ		INT8_15_Handler
    IRQ  	DT_Handler
    IRQ  	MFS0RX_IRQHandler
    IRQ  	MFS0TX_IRQHandler
    IRQ  	MFS1RX_IRQHandler
    IRQ  	MFS1TX_IRQHandler
    IRQ  	MFS2RX_IRQHandler
    IRQ  	MFS2TX_IRQHandler
    IRQ  	MFS3RX_IRQHandler
    IRQ  	MFS3TX_IRQHandler
    IRQ  	MFS4RX_IRQHandler
    IRQ  	MFS4TX_IRQHandler
    IRQ  	MFS5RX_IRQHandler
    IRQ  	MFS5TX_IRQHandler
    IRQ  	MFS6RX_IRQHandler
    IRQ  	MFS6TX_IRQHandler
    IRQ  	MFS7RX_IRQHandler
    IRQ  	MFS7TX_IRQHandler
    IRQ  	PPG_Handler
    IRQ  	TIM_IRQHandler
    IRQ  	ADC0_IRQHandler
    IRQ  	ADC1_IRQHandler
    IRQ  	ADC2_IRQHandler
    IRQ  	MFT_FRT_IRQHandler
    IRQ  	MFT_IPC_IRQHandler
    IRQ 	MFT_OPC_IRQHandler
    IRQ  	BT_IRQHandler
    IRQ  	CAN0_IRQHandler
    IRQ  	CAN1_IRQHandler
    IRQ  	USBF_Handler
    IRQ  	USB_Handler
    IRQ  	DummyHandler_1
    IRQ  	DummyHandler_2
    IRQ  	DMAC0_Handler
    IRQ  	DMAC1_Handler
    IRQ  	DMAC2_Handler
    IRQ  	DMAC3_Handler
    IRQ  	DMAC4_Handler
    IRQ  	DMAC5_Handler
    IRQ  	DMAC6_Handler
    IRQ 	DMAC7_Handler
    IRQ  	DummyHandler_3
    IRQ  	DummyHandler_4

    .end
