/*=============================================================================
    Startup code - Interrupt table definition, data initialization
    
    CPU:  Spansion MB9BFD1x
    Tool: generic
    File: startup_mb9bfdxx.s
    
    Version:  1.0
    Date:     11/24/2014
    
  -----------------------------------------------------------------------------
  Copyright (c)2013 iSYSTEM AG, All rights reserved.
  -----------------------------------------------------------------------------
  Software that is described herein is for illustrative purposes only which 
  provides customers with programming information regarding the iSYSTEM products.  
  This software is supplied "AS IS" without any warranties of any kind, and 
  iSYSTEM and its licensor disclaim any and all warranties, express or implied, 
  including all implied warranties of merchantability, fitness for a particular 
  purpose and non-infringement of intellectual property rights.  
  iSYSTEM assumes no responsibility or liability for the use of the software, 
  conveys no license or rights under any patent, copyright, mask work right, 
  or any other intellectual property rights in or to any products. 
  iSYSTEM reserves the right to make changes in the software without notification. 
  iSYSTEM also makes no representation or warranty that such application will be 
  suitable for the specified use without further testing or modification.
  
  Permission to use, copy, modify, and distribute this software and its
  documentation is hereby granted, under iSYSTEMs' and its licensor's relevant 
  copyrights in the software, without fee, provided that it is used in conjunction 
  with iSYSTEM products. This copyright, permission, and disclaimer notice 
  must appear in all copies of this code.
  =============================================================================*/
/* 
  The following symbols are provide by the linker definition file
*/
extern unsigned long _etext;        // End of code (text) section
extern unsigned long _sdata;        // Start of data section
extern unsigned long _edata;        // End of data section
extern unsigned long _sbss;         // Start of bss (zero init) data section
extern unsigned long _ebss;         // End of bss (zero init) data section
extern unsigned long _sidata;       // Start of initialized data section
extern unsigned long _estack;       // Stack top - end of RAM
extern unsigned long _isr_vector;   // Start of ISR Vector table

/*=============================================================================
  Forward declaration(s) - Reset Handler, Default Handler etc.
  =============================================================================*/
void Reset_Handler(void);
void Default_Handler(void);

/*=============================================================================
  External functions 
  =============================================================================*/ 
extern void SystemInit();
extern void main(void);

/*=============================================================================
  Reset Handler
 ============================================================================= */
__attribute__((naked)) void Reset_Handler()
{
	unsigned long *pSrcData, *pDestData;

	/* Copy initialized data from Flash to RAM */
	pSrcData = &_sidata;
    for(pDestData = &_sdata; pDestData < &_edata; )
    {
        *(pDestData++) = *(pSrcData++);
    }

	/* Fill/init the bss section with zeros */
	for(pDestData = &_sbss; pDestData < &_ebss; )
    {
        *(pDestData++) = 0;
    }

    /* Set the stack pointer (R13) to point to the end of RAM */
	asm("ldr sp,=_estack\n\t");

    /* Set the vector table offset register */
	pDestData  = (unsigned long *)0xe000ed08;
	*pDestData = (unsigned long)&_isr_vector;
	
	/* Run static constructors */
  /*	__libc_init_array(); */

	/* Call the clock system initialization function.*/
	SystemInit();

	/* Call main */
	main();

	/* If main() ever returns call */
	Default_Handler();
}

/*===========================================================================
  Forward declaration and references to ISR - 
  Defined weak to point to Default_Handler in case no specific implementation
  is provided (which would overide the weak) definition
  ===========================================================================*/
  /* Generic Cortex-Mx Exceoptions */
void NMI_Handler() __attribute__ ((weak, alias("Default_Handler")));
void HardFault_Handler() __attribute__ ((weak, alias("Default_Handler")));
void MemManage_Handler() __attribute__ ((weak, alias("Default_Handler")));
void BusFault_Handler() __attribute__ ((weak, alias("Default_Handler")));
void UsageFault_Handler() __attribute__ ((weak, alias("Default_Handler")));
void SVC_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DebugMonitor_Handler() __attribute__ ((weak, alias("Default_Handler")));
void PendSV_Handler() __attribute__ ((weak, alias("Default_Handler")));
void SysTick_Handler() __attribute__ ((weak, alias("Default_Handler")));

  /* Device specific Interrupts */
void CSV_Handler() __attribute__ ((weak, alias("Default_Handler")));
void SWDT_Handler() __attribute__ ((weak, alias("Default_Handler")));
void LVD_Handler() __attribute__ ((weak, alias("Default_Handler")));
void MFT_WG_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void INT0_7_Handler() __attribute__ ((weak, alias("Default_Handler")));
void INT8_31_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DT_Handler() __attribute__ ((weak, alias("Default_Handler")));
void MFS0RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS0TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS1RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS1TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS2RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS2TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS3RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS3TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS4RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS4TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS5RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS5TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS6RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS6TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS7RX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFS7TX_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void PPG_Handler() __attribute__ ((weak, alias("Default_Handler")));
void TIM_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void ADC0_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void ADC1_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void ADC2_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFT_FRT_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFT_IPC_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void MFT_OPC_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void BT0_7_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void ETHER0_CAN0_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void ETHER1_CAN1_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
void USB0F_Handler() __attribute__ ((weak, alias("Default_Handler")));
void USB0_Handler() __attribute__ ((weak, alias("Default_Handler")));
void USB1F_Handler() __attribute__ ((weak, alias("Default_Handler")));
void USB1_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC0_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC1_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC2_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC3_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC4_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC5_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC6_Handler() __attribute__ ((weak, alias("Default_Handler")));
void DMAC7_Handler() __attribute__ ((weak, alias("Default_Handler")));
void BT8_15_IRQHandler() __attribute__ ((weak, alias("Default_Handler")));
/*=============================================================================
  Interrupt vector table - 0 to 15 are Cortex M generic
  =============================================================================*/
  
void (* const InterruptVector[])() __attribute__ ((section(".isr_vector"))) = {
		/* Generic Cortex M exceptions o to 15*/
		(void (*)(void))&_estack, /* 0 Stack pointer */
		Reset_Handler,            /* 1 Reset */
		NMI_Handler,              /* 2 NMI  */
		HardFault_Handler,        /* 3 Hard fault */
		MemManage_Handler,        /* 4 Memory management fault */
		BusFault_Handler,         /* 5 Bus fault */
		UsageFault_Handler,       /* 6 Usage fault */
		0,                        /* 7 Reserved */
		0,                        /* 8 Reserved */
		0,                        /* 9 Reserved */
		0,                        /* 10 Reserved */
		SVC_Handler,              /* 11 SVCall */
		DebugMonitor_Handler,     /* 12 Reserved for Debug */
		0,                        /* 13 Reserved */
		PendSV_Handler,           /* 14 PendSV */
		SysTick_Handler,          /* 15 Systick */

		/* Device specific exeptions 16 to 255*/
		CSV_Handler,              /* 0: Clock Super Visor */
		SWDT_Handler,             /* 1: Software Watchdog Timer */
		LVD_Handler,              /* 2: Low Voltage Detector */
		MFT_WG_IRQHandler,        /* 3: Wave Form Generator / DTIF */
		INT0_7_Handler,           /* 4: External Interrupt Request ch.0 to ch.7 */
		INT8_31_Handler,          /* 5: External Interrupt Request ch.8 to ch.31 */
		DT_Handler,               /* 6: Dual Timer / Quad Decoder */
		MFS0RX_IRQHandler,        /* 7: MultiFunction Serial ch.0 */
		MFS0TX_IRQHandler,        /* 8: MultiFunction Serial ch.0 */
		MFS1RX_IRQHandler,        /* 9: MultiFunction Serial ch.1 */
		MFS1TX_IRQHandler,        /* 10: MultiFunction Serial ch.1 */
		MFS2RX_IRQHandler,        /* 11: MultiFunction Serial ch.2 */
		MFS2TX_IRQHandler,        /* 12: MultiFunction Serial ch.2 */
		MFS3RX_IRQHandler,        /* 13: MultiFunction Serial ch.3 */
		MFS3TX_IRQHandler,        /* 14: MultiFunction Serial ch.3 */
		MFS4RX_IRQHandler,        /* 15: MultiFunction Serial ch.4 */
		MFS4TX_IRQHandler,        /* 16: MultiFunction Serial ch.4 */
		MFS5RX_IRQHandler,        /* 17: MultiFunction Serial ch.5 */
		MFS5TX_IRQHandler,        /* 18: MultiFunction Serial ch.5 */
		MFS6RX_IRQHandler,        /* 19: MultiFunction Serial ch.6 */
		MFS6TX_IRQHandler,        /* 20: MultiFunction Serial ch.6 */
		MFS7RX_IRQHandler,        /* 21: MultiFunction Serial ch.7 */
		MFS7TX_IRQHandler,        /* 22: MultiFunction Serial ch.7 */
		PPG_Handler,              /* 23: PPG */
		TIM_IRQHandler,           /* 24: OSC / PLL / Watch Counter */
		ADC0_IRQHandler,          /* 25: ADC0 */
		ADC1_IRQHandler,          /* 26: ADC1 */
		ADC2_IRQHandler,          /* 27: ADC2 */
		MFT_FRT_IRQHandler,       /* 28: Free-run Timer */
		MFT_IPC_IRQHandler,       /* 29: Input Capture */
		MFT_OPC_IRQHandler,       /* 30: Output Compare */
		BT0_7_IRQHandler,         /* 31: Base Timer ch.0 to ch.7 */
		ETHER0_CAN0_IRQHandler,   /* 32: Ethernet MAC ch.0 / CAN ch.0 */
		ETHER1_CAN1_IRQHandler,   /* 33: Ethernet MAC ch.1 / CAN ch.1 */
		USB0F_Handler,            /* 34: USB0 Function */
		USB0_Handler,             /* 35: USB0 Function / USB0 HOST */
		USB1F_Handler,            /* 36: USB1 Function */
		USB1_Handler,             /* 37: USB1 Function / USB1 HOST */
		DMAC0_Handler,            /* 38: DMAC ch.0 */
		DMAC1_Handler,            /* 39: DMAC ch.1 */
		DMAC2_Handler,            /* 40: DMAC ch.2 */
		DMAC3_Handler,            /* 41: DMAC ch.3 */
		DMAC4_Handler,            /* 42: DMAC ch.4 */
		DMAC5_Handler,            /* 43: DMAC ch.5 */
		DMAC6_Handler,            /* 44: DMAC ch.6 */
		DMAC7_Handler,            /* 45: DMAC ch.7 */
		BT8_15_IRQHandler,        /* 46: Base Timer ch.8 to ch.15 */
		Default_Handler           /* 47: Reserved */
};


/*=============================================================================
  Default Interrupt Handler
  =============================================================================*/
void Default_Handler()
{
	while(1)
	{
	}
}
